/*
  Main file of the app, initializing stuff and setting the routes up.
*/

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:local_session_timeout/local_session_timeout.dart';
import 'package:pref/pref.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/zebra_scan.dart';
import 'package:snipe_scanner/widgets/audit.dart';
import 'package:snipe_scanner/widgets/bulk_checkin.dart';
import 'package:snipe_scanner/widgets/bulk_checkout.dart';
import 'package:snipe_scanner/widgets/checkin_asset.dart';
import 'package:snipe_scanner/widgets/checkout_asset.dart';
import 'package:snipe_scanner/widgets/dashboard.dart';
import 'package:snipe_scanner/widgets/asset_detail.dart';
import 'package:snipe_scanner/widgets/login.dart';
import 'package:snipe_scanner/widgets/search_asset.dart';
import 'package:snipe_scanner/widgets/user_detail.dart';
import 'package:snipe_scanner/widgets/preferences.dart';
import 'utils/logging.dart';

import 'model/app_state.dart';

// navigator key is used to navigate through the app programatically even without being inside the app state.
final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();
// route observer is used to observe navigating through the app, it offers functionality for the views to know if they are navigated to..
final RouteObserver<PageRoute> routeObserver = RouteObserver<PageRoute>();
// shared preference service, this holds the preferences and persists them on the phone.
late final PrefServiceShared service;

Future main() async {
  WidgetsFlutterBinding.ensureInitialized();

  service = await PrefServiceShared.init(defaults: {
    'host': '',
    'custom_user_url': '',
    'use_custom_user_url': false,
    'tag_prefix': '',
    'use_tag': true,
    'auto_logout': false,
    'auto_logout_time': 60,
    'warn_before_checkin_from_asset': false
  });
  // init the zebra scanning engine and setup the datawedge profile
  ZebraScan.init();
  // init the audio players for the scan responses
  initAudio();
  // init the remote logging for debugging purposes
  initLogger(const MyApp(), navigatorKey);
  runApp(PrefService(
      service: service,
      child: ChangeNotifierProvider(
          create: (_) => AppState(), child: const MyApp())));
}

// this is the main app widget that gets inserted into the app, it's the startingpoint for everything
class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  // This widget is the root of the app.
  @override
  Widget build(BuildContext context) {
    Duration? timeoutDuration;
    if (service.get('auto_logout')) {
      timeoutDuration = Duration(minutes: service.get('auto_logout_time'));
    }
    final sessionConfig =
        SessionConfig(invalidateSessionForAppLostFocus: timeoutDuration);
    sessionConfig.stream.listen((SessionTimeoutState timeoutEvent) {
      if (timeoutEvent == SessionTimeoutState.appFocusTimeout) {
        navigatorKey.currentState?.popUntil((route) => route.isFirst);
      }
    });
    // set the orientation to always be portrait
    SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
    // initialize the material app
    return SessionTimeoutManager(
        sessionConfig: sessionConfig,
        child: MaterialApp(
          title: 'SnipeIT Scanner',
          debugShowCheckedModeBanner: false,
          navigatorKey: navigatorKey,
          theme: ThemeData(
              primaryColor: const Color(0xFF2661FA),
              scaffoldBackgroundColor: Colors.white,
              visualDensity: VisualDensity.adaptivePlatformDensity,
              bottomSheetTheme: const BottomSheetThemeData(
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadiusDirectional.all(Radius.zero)))),
          navigatorObservers: [routeObserver],
          // this defines the first view.
          home: const LoginWidget(),
          // here are all the views within the app, so you can navigate through them using named routes
          routes: <String, WidgetBuilder>{
            '/dashboard': (BuildContext context) => const DashboardWidget(),
            '/assetDetail': (BuildContext context) => const AssetDetail(),
            '/checkoutAsset': (BuildContext context) => const CheckoutAsset(),
            '/checkinAsset': (BuildContext context) => const CheckinAsset(),
            '/bulkCheckin': (BuildContext context) => const BulkCheckin(),
            '/bulkCheckout': (BuildContext context) => const BulkCheckout(),
            '/searchAsset': (BuildContext context) => const SearchAsset(),
            '/userDetail': (BuildContext context) => const UserDetail(),
            '/auditAsset': (BuildContext context) => const Audit(),
            '/preferences': (BuildContext context) => const PreferencesWidget(),
          },
        ));
  }
}
