/* 
  Here are all the http requests to handle assets
*/
import 'dart:convert';
import 'dart:io';

import 'package:snipe_scanner/main.dart';
import 'package:snipe_scanner/model/activity.dart';
import 'package:snipe_scanner/model/asset.dart';
import 'package:http/http.dart' as http;
import 'package:snipe_scanner/model/response_status.dart';
import 'package:snipe_scanner/service/login_user.dart';

Future<Asset> getAssetFromTag(String token, String tag) async {
  final response = await http.get(
      Uri.parse("${service.get('host')}/api/v1/hardware/bytag/$tag"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json; charset=UTF-8',
        HttpHeaders.acceptHeader: 'application/json',
      });
  if (response.statusCode != 200) {
    throw Error();
  }

  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw SnipeException(responseJson["messages"].toString());
  }
  if (responseJson["rows"] != null) {
    return Asset.fromJson(responseJson["rows"][0]);
  }
  return Asset.fromJson(responseJson);
}

Future<List<Asset>> getAssets(String token, [String search = ""]) async {
  final response = await http.get(
      Uri.parse("${service.get('host')}/api/v1/hardware?search=$search"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json; charset=UTF-8',
        HttpHeaders.acceptHeader: 'application/json',
      });
  if (response.statusCode != 200) {
    throw Error();
  }

  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  List<Asset> assets =
      (responseJson["rows"] as List).map((i) => Asset.fromJson(i)).toList();
  return assets;
}

Future<List<Asset>> getUserAssets(String token, int id) async {
  final response = await http.get(
      Uri.parse("${service.get('host')}/api/v1/users/$id/assets"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json; charset=UTF-8',
        HttpHeaders.acceptHeader: 'application/json',
      });
  if (response.statusCode != 200) {
    throw Error();
  }

  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  List<Asset> assets =
      (responseJson["rows"] as List).map((i) => Asset.fromJson(i)).toList();
  return assets;
}

Future<List<Activity>> getAssetActivity(String token, int id) async {
  final response = await http.get(
      Uri.parse(
          "${service.get('host')}/api/v1/reports/activity?item_type=asset&item_id=${id.toString()}"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json',
        HttpHeaders.acceptHeader: 'application/json',
      });
  if (response.statusCode != 200) {
    throw Error();
  }
  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  List<Activity> activities =
      (responseJson["rows"] as List).map((i) => Activity.fromJson(i)).toList();
  activities.sort((u, v) {
    return u.id!.compareTo(v.id!);
  });
  return activities;
}

Future<ResponseStatus> checkinAsset(String token, int id,
    {String note = "", int? locationId}) async {
  var body = jsonEncode({'note': note, 'location_id': locationId});
  final response = await http.post(
      Uri.parse(
          "${service.get('host')}/api/v1/hardware/${id.toString()}/checkin"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json; charset=UTF-8',
        HttpHeaders.acceptHeader: 'application/json',
      },
      body: body);
  if (response.statusCode != 200) {
    throw Error();
  }
  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  return ResponseStatus.fromJson(responseJson);
}

Future<ResponseStatus> checkoutAsset(String token, int id, int dstId,
    {String note = "", String type = "user"}) async {
  var o = <String, dynamic>{'checkout_to_type': type, 'note': note};
  if (type == "user") {
    o["assigned_user"] = dstId;
  } else if (type == "location") {
    o["assigned_location"] = dstId;
  } else if (type == "asset") {
    o["assigned_asset"] = dstId;
  } else {
    throw Exception("wrong type");
  }
  var body = jsonEncode(o);
  final response = await http.post(
      Uri.parse(
          "${service.get('host')}/api/v1/hardware/${id.toString()}/checkout"),
      headers: {
        HttpHeaders.authorizationHeader: 'Bearer $token',
        HttpHeaders.contentTypeHeader: 'application/json',
        HttpHeaders.acceptHeader: 'application/json',
      },
      body: body);
  if (response.statusCode != 200) {
    throw Error();
  }
  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  return ResponseStatus.fromJson(responseJson);
}

Future<ResponseStatus> auditAsset(String token, String tag,
    {String note = "", int dstId = -1}) async {
  var o = <String, dynamic>{'note': note, 'asset_tag': tag};
  if (dstId >= 0) {
    o['location_id'] = dstId;
    o['update_location'] = 1;
  }
  var body = jsonEncode(o);
  final response =
      await http.post(Uri.parse("${service.get('host')}/api/v1/hardware/audit"),
          headers: {
            HttpHeaders.authorizationHeader: 'Bearer $token',
            HttpHeaders.contentTypeHeader: 'application/json; charset=UTF-8',
            HttpHeaders.acceptHeader: 'application/json',
          },
          body: body);
  if (response.statusCode != 200) {
    throw Error();
  }
  final responseJson = jsonDecode(utf8.decode(response.bodyBytes));
  if (responseJson["status"].toString() == "error") {
    throw Exception(responseJson["messages"].toString());
  }
  return ResponseStatus.fromJson(responseJson);
}
