import 'package:flutter/material.dart';
import 'package:html_unescape/html_unescape.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/history_row.dart';
import 'package:snipe_scanner/components/text_row.dart';
import 'package:snipe_scanner/model/activity.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/model/asset.dart';
import 'package:snipe_scanner/service/asset.dart';
import 'package:snipe_scanner/service/login_user.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

class AssetHistoryView extends StatelessWidget {
  final List<Activity> history;
  const AssetHistoryView({super.key, required this.history});

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Column(
      children: [
        SizedBox(height: size.height * 0.03),
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
          child: const Text(
            "History",
            style: TextStyle(fontWeight: FontWeight.bold, fontSize: 24),
          ),
        ),
        Expanded(
          child: Scrollbar(
              child: ListView.builder(
                  shrinkWrap: true,
                  itemCount: history.length,
                  itemBuilder: (BuildContext context, int index) {
                    var i = history.length - (index + 1);
                    return historyRow(history[i], context);
                  })),
        )
      ],
    );
  }
}

class AssetGeneralView extends StatelessWidget {
  final Asset asset;
  const AssetGeneralView({super.key, required this.asset});

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Column(
      children: [
        if (asset.image != null)
          Container(
              height: size.height * 0.3,
              padding: const EdgeInsets.only(top: 20, bottom: 40),
              child: Image(image: NetworkImage(asset.image!))),
        Expanded(
          child: SingleChildScrollView(
            child: Column(
              children: [
                TextRow(title: "Name", text: asset.name ?? " - "),
                TextRow(title: "ID", text: asset.id.toString()),
                TextRow(title: "Tag", text: asset.assetTag!),
                TextRow(
                    title: "Last Audit",
                    text: asset.lastAuditDate?.formatted ?? " - "),
                if (asset.assignedTo != null)
                  TextRow(
                      title: "Checked out to",
                      text:
                          "${asset.assignedTo!.name!}\n(${asset.lastCheckout!.formatted})"),
                if (asset.assignedTo == null)
                  const TextRow(title: "Checked out to", text: " - "),
                TextRow(
                    title: "Notes",
                    text: HtmlUnescape().convert(asset.notes ?? " - ")),
              ],
            ),
          ),
        )
      ],
    );
  }
}

class AssetDetail extends StatefulWidget {
  const AssetDetail({super.key});

  @override
  ScanAwareState<AssetDetail> createState() => _AssetDetailState();
}

class _AssetDetailState extends ScanAwareState<AssetDetail> {
  int _index = 0;
  String _error = "";
  bool _loaded = false;
  Asset? _asset;

  @override
  void onScan(barcode) {
    playSuccess();
    Navigator.pushReplacementNamed(context, "/assetDetail", arguments: barcode);
  }

  @override
  void loadData() async {
    final args = ModalRoute.of(context)!.settings.arguments as String;
    var token = Provider.of<AppState>(context, listen: false).token;

    setState(() {
      _loaded = false;
      _error = "";
    });
    Asset asset;
    try {
      asset = await getAssetFromTag(token, args);
    } on SnipeException catch (err) {
      setState(() {
        _error = err.toString();
        _loaded = true;
      });
      return;
    }
    if (asset.image != null && context.mounted) {
      await precacheImage(NetworkImage(asset.image!), context);
    }
    setState(() {
      _asset = asset;
      _loaded = true;
    });
  }

  Widget _selector(BuildContext context) {
    if (_index == 1) {
      return FutureBuilder<List<Activity>>(
          future: getAssetActivity(
              Provider.of<AppState>(context, listen: false).token, _asset!.id),
          builder:
              (BuildContext context, AsyncSnapshot<List<Activity>> snapshot) {
            if (snapshot.hasData) {
              return AssetHistoryView(history: snapshot.data!);
            } else if (snapshot.hasError) {
              return Container(
                  padding:
                      const EdgeInsets.symmetric(horizontal: 40, vertical: 40),
                  child: Text(
                    _error,
                    style: const TextStyle(fontSize: 20),
                  ));
            } else {
              return const Center(child: CircularProgressIndicator());
            }
          });
    }

    return AssetGeneralView(asset: _asset!);
  }

  @override
  Widget build(BuildContext context) {
    if (!_loaded) {
      return const Center(
        child: CircularProgressIndicator(),
      );
    }
    if (_error != "") {
      return Scaffold(
          appBar: AppBar(
            title: const Text("Fehler"),
            actions: [
              IconButton(
                  icon: const Icon(Icons.refresh),
                  tooltip: 'Reload',
                  onPressed: () => loadData()),
            ],
          ),
          body: Container(
            padding: const EdgeInsets.symmetric(horizontal: 40, vertical: 40),
            child: Text(
              _error,
              style: const TextStyle(fontSize: 20),
            ),
          ));
    }
    return Scaffold(
        appBar: AppBar(
          title: Text(_asset!.getShortName()),
          actions: [
            if (_asset!.availableActions!.checkout &&
                _asset!.assignedTo == null)
              IconButton(
                  icon: const Icon(Icons.north_east),
                  tooltip: 'Checkout',
                  onPressed: () {
                    Navigator.pushNamed(context, "/checkoutAsset",
                        arguments: _asset!);
                  }),
            if (_asset!.availableActions!.checkin && _asset!.assignedTo != null)
              IconButton(
                  icon: const Icon(Icons.south_west),
                  tooltip: 'Checkin',
                  onPressed: () {
                    Navigator.pushNamed(context, "/checkinAsset",
                        arguments: _asset!);
                  }),
            IconButton(
                icon: const Icon(Icons.refresh),
                tooltip: 'Reload',
                onPressed: () {
                  loadData();
                }),
          ],
        ),
        bottomNavigationBar: BottomNavigationBar(
          currentIndex: _index,
          items: const <BottomNavigationBarItem>[
            BottomNavigationBarItem(
              icon: Icon(Icons.home),
              label: 'General',
            ),
            BottomNavigationBarItem(
              icon: Icon(Icons.list),
              label: 'History',
            ),
          ],
          onTap: (value) {
            setState(() {
              _index = value;
            });
          },
        ),
        body: _selector(context));
  }
}
