import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/asset_row.dart';
import 'package:snipe_scanner/components/manual_input.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/model/asset.dart';
import 'package:snipe_scanner/service/asset.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

import '../main.dart';

class BulkCheckin extends StatefulWidget {
  const BulkCheckin({super.key});

  @override
  ScanAwareState<BulkCheckin> createState() => _BulkCheckinState();
}

class _BulkCheckinState extends ScanAwareState<BulkCheckin> {
  final List<dynamic> _history = [];
  final TextEditingController _noteController = TextEditingController();
  bool _keepNote = false;
  bool _showCheckinAlert = false;
  bool _ignoreCheckinAlertAction = false;
  bool _ignoreCheckinAlert = false;
  String _error = "";
  Asset? _checkinAlertAsset;
  Map<String, dynamic>? _checkinAlertItem;

  @override
  void dispose() {
    _noteController.dispose();
    super.dispose();
  }

  void _cancelCheckin() {
    setState(() {
      _showCheckinAlert = false;
      _checkinAlertAsset = null;
      if (_checkinAlertItem != null) {
        _checkinAlertItem!["status"] = false;
      }
      _checkinAlertItem = null;
    });
  }

  void _confirmCheckin() {
    var asset = _checkinAlertAsset!;
    var item = _checkinAlertItem!;
    setState(() {
      _showCheckinAlert = false;
      _checkinAlertAsset = null;
      _checkinAlertItem = null;
    });
    var token = Provider.of<AppState>(context, listen: false).token;
    _checkinFunc(token, asset, item);
  }

  @override
  void onScan(String barcode) {
    _checkinAsset(barcode);
  }

  void _checkinAsset(String barcode) async {
    _cancelCheckin();
    setState(() {
      _error = "";
    });
    var token = Provider.of<AppState>(context, listen: false).token;
    Map<String, dynamic> item = {"barcode": barcode};
    setState(() {
      _history.add(item);
    });
    Asset asset;
    try {
      asset = await getAssetFromTag(token, barcode);
    } catch (err) {
      setState(() {
        _error = "Asset not found";
        item["status"] = false;
      });
      playFail();
      return;
    }
    item["id"] = asset.id;
    if (asset.name != null) {
      setState(() {
        if (asset.assignedTo != null) {
          item["checkinFrom"] = asset.assignedTo!.name;
        }
        item["name"] = asset.getShortName();
      });
    }
    if (asset.assignedTo != null &&
        asset.assignedTo!.type == 'asset' &&
        service.get('warn_before_checkin_from_asset') as bool) {
      if (!_ignoreCheckinAlert) {
        setState(() {
          _showCheckinAlert = true;
          _checkinAlertAsset = asset;
          _checkinAlertItem = item;
        });
        return;
      } else if (!_ignoreCheckinAlertAction) {
        item["status"] = false;
        return;
      }
    }
    _checkinFunc(token, asset, item);
  }

  void _checkinFunc(
      String token, Asset asset, Map<String, dynamic> item) async {
    try {
      await checkinAsset(token, asset.id, note: _noteController.text);
    } catch (err) {
      setState(() {
        _error = err.toString();
        item["status"] = false;
      });
      playFail();
      return;
    }
    playSuccess();
    setState(() {
      item["note"] = _noteController.text;
      item["status"] = true;
      if (!_keepNote) _noteController.text = "";
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text("Bulk checkin"),
        actions: [
          IconButton(
              icon: const Icon(Icons.search),
              tooltip: 'Search for Assets',
              onPressed: () async {
                final result =
                    await Navigator.pushNamed(context, '/searchAsset');
                if (result != null) {
                  _checkinAsset('$result');
                }
              }),
        ],
      ),
      body: Stack(children: [
        Column(
          children: [
            ManualInput(cb: _checkinAsset),
            Container(
              margin: const EdgeInsets.symmetric(horizontal: 40),
              child: TextField(
                  decoration: const InputDecoration(labelText: "Note"),
                  controller: _noteController),
            ),
            SizedBox(
                height: 50,
                child: Row(
                  children: [
                    Container(
                      margin: const EdgeInsets.only(left: 40, right: 5),
                      child: Checkbox(
                        value: _keepNote,
                        onChanged: (value) {
                          if (value != null) {
                            setState(() {
                              _keepNote = value;
                            });
                          }
                        },
                      ),
                    ),
                    const Text("Keep note")
                  ],
                )),
            Container(
              padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
              child: const Text(
                "History",
                style: TextStyle(fontWeight: FontWeight.bold, fontSize: 24),
              ),
            ),
            Expanded(
              child: Scrollbar(
                  child: ListView.builder(
                      itemCount: _history.length,
                      itemBuilder: (BuildContext context, int index) {
                        var i = _history.length - (index + 1);
                        return assetRow(_history[i]["barcode"], context,
                            name: _history[i]["name"],
                            note: _history[i]["note"],
                            status: _history[i]["status"],
                            checkinFrom: _history[i]["checkinFrom"]);
                      })),
            )
          ],
        ),
        if (_showCheckinAlert)
          AlertDialog(
            title: const Text('Are you sure?'),
            content: const Text(
                'Do you really want to check in this asset from another asset? Maybe it belongs to a set.'),
            actions: [
              Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Checkbox(
                    onChanged: (bool? value) {
                      setState(() {
                        _ignoreCheckinAlert = value!;
                      });
                    },
                    value: _ignoreCheckinAlert,
                    activeColor: const Color(0xFF6200EE),
                  ),
                  const Text('Don\'t ask again'),
                ],
              ),
              TextButton(
                style: TextButton.styleFrom(
                    textStyle: const TextStyle(
                  color: Color(0xFF6200EE),
                )),
                onPressed: () {
                  if (_ignoreCheckinAlert) {
                    _ignoreCheckinAlertAction = false;
                  }
                  _cancelCheckin();
                },
                child: const Text('Cancel'),
              ),
              TextButton(
                style: TextButton.styleFrom(
                    textStyle: const TextStyle(
                  color: Color(0xFF6200EE),
                )),
                onPressed: () {
                  if (_ignoreCheckinAlert) {
                    _ignoreCheckinAlertAction = true;
                  }
                  _confirmCheckin();
                },
                child: const Text('Yes!'),
              ),
            ],
          ),
        if (_error != "")
          AlertDialog(
            title: const Text('Error'),
            content: Text(_error),
            actions: [
              TextButton(
                style: TextButton.styleFrom(
                    textStyle: const TextStyle(
                  color: Color(0xFF6200EE),
                )),
                onPressed: () {
                  setState(() {
                    _error = "";
                  });
                },
                child: const Text('OK'),
              ),
            ],
          ),
      ]),
    );
  }
}
