import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/asset_row.dart';
import 'package:snipe_scanner/components/manual_input.dart';
import 'package:snipe_scanner/components/target_selector.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/model/asset.dart';
import 'package:snipe_scanner/service/asset.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

class BulkCheckout extends StatefulWidget {
  const BulkCheckout({super.key});

  @override
  ScanAwareState<BulkCheckout> createState() => _BulkCheckoutState();
}

class _BulkCheckoutState extends ScanAwareState<BulkCheckout> {
  final List<dynamic> _history = [];
  final TextEditingController _noteController = TextEditingController();

  String _type = "user";
  int _dst = 0;
  bool _keepNote = false;
  String _error = "";

  @override
  void dispose() {
    _noteController.dispose();
    super.dispose();
  }

  @override
  void onScan(String barcode) async {
    setState(() {
      _error = "";
    });
    var token = Provider.of<AppState>(context, listen: false).token;
    Map<String, dynamic> item = {"barcode": barcode};
    setState(() {
      _history.add(item);
    });
    Asset asset;
    try {
      asset = await getAssetFromTag(token, barcode);
    } catch (err) {
      setState(() {
        _error = "Asset not found";
        item["status"] = false;
      });
      playFail();
      return;
    }
    if (asset.name != null) {
      setState(() {
        if (asset.assignedTo != null) {
          item["checkoutFrom"] = asset.assignedTo!.name;
        }
        item["name"] = asset.getShortName();
      });
    }
    try {
      await checkoutAsset(token, asset.id, _dst,
          note: _noteController.text, type: _type);
    } catch (err) {
      setState(() {
        _error = err.toString();
        item["status"] = false;
      });
      playFail();
      return;
    }
    playSuccess();
    setState(() {
      item["note"] = _noteController.text;
      item["status"] = true;
      if (!_keepNote) _noteController.text = "";
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text("Bulk checkout"),
        actions: [
          IconButton(
              icon: const Icon(Icons.search),
              tooltip: 'Search for Assets',
              onPressed: () async {
                final result =
                    await Navigator.pushNamed(context, '/searchAsset');
                if (result != null) {
                  onScan('$result');
                }
              }),
        ],
      ),
      body: Stack(children: [
        Column(
          children: [
            TargetSelector(cb: (type, id) {
              _dst = id;
              _type = type;
            }),
            ManualInput(cb: onScan),
            Container(
              margin: const EdgeInsets.symmetric(horizontal: 40),
              child: TextField(
                  decoration: const InputDecoration(labelText: "Note"),
                  controller: _noteController),
            ),
            SizedBox(
                height: 34,
                child: Row(
                  children: [
                    Container(
                      margin: const EdgeInsets.only(left: 40, right: 5),
                      child: Checkbox(
                        value: _keepNote,
                        onChanged: (value) {
                          if (value != null) {
                            setState(() {
                              _keepNote = value;
                            });
                          }
                        },
                      ),
                    ),
                    const Text("Keep note")
                  ],
                )),
            Container(
              padding: const EdgeInsets.fromLTRB(0, 0, 0, 4),
              child: const Text(
                "History",
                style: TextStyle(fontWeight: FontWeight.bold, fontSize: 24),
              ),
            ),
            Expanded(
              child: Scrollbar(
                  child: ListView.builder(
                      itemCount: _history.length,
                      itemBuilder: (BuildContext context, int index) {
                        var i = _history.length - (index + 1);
                        return assetRow(_history[i]["barcode"], context,
                            name: _history[i]["name"],
                            note: _history[i]["note"],
                            status: _history[i]["status"]);
                      })),
            )
          ],
        ),
        if (_error != "")
          AlertDialog(
            title: const Text('Error'),
            content: Text(_error),
            actions: [
              TextButton(
                style: TextButton.styleFrom(
                    textStyle: const TextStyle(
                  color: Color(0xFF6200EE),
                )),
                onPressed: () {
                  setState(() {
                    _error = "";
                  });
                },
                child: const Text('OK'),
              ),
            ],
          ),
      ]),
    );
  }
}
