
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/big_button.dart';
import 'package:snipe_scanner/components/build_info.dart';
import 'package:snipe_scanner/components/manual_input.dart';
import 'package:snipe_scanner/components/user_selector.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/service/login_user.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

class DashboardWidget extends StatefulWidget {
  const DashboardWidget({super.key});

  @override
  ScanAwareState<DashboardWidget> createState() => _DashboardWidgetState();
}

class _DashboardWidgetState extends ScanAwareState<DashboardWidget> {
  final ScrollController _scrollController = ScrollController();
  String _username = "";
  String _avatar = "";
  bool loaded = false;

  @override
  void onScan(String barcode) {
    playSuccess();
    Navigator.pushNamed(context, "/assetDetail", arguments: barcode);
  }

  // Bin zu Dashboard hin navigiert.
  @override
  void didPush() {
    FocusScope.of(context).unfocus();
    if (_scrollController.hasClients) {
      _scrollController.animateTo(0,
          duration: const Duration(milliseconds: 500), curve: Curves.easeIn);
    }
    super.didPush();
  }

  // Bin zu Dashboard zurück gekommen.
  @override
  void didPopNext() {
    FocusScope.of(context).unfocus();
    if (_scrollController.hasClients) {
      _scrollController.animateTo(0,
          duration: const Duration(milliseconds: 500), curve: Curves.easeIn);
    }
    super.didPopNext();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  void loadData() async {
    var token = Provider.of<AppState>(context, listen: false).token;
    var user = await getUser(token);
    var avatar = "";
    if (user.avatar.startsWith("//")) {
      avatar = "https:${user.avatar}";
    } else {
      avatar = user.avatar;
    }
    if (context.mounted) {
      await precacheImage(NetworkImage(avatar), context);
    }
    setState(() {
      _username = user.name;
      _avatar = avatar;
      loaded = true;
    });
  }

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    if (!loaded) {
      return Container(
        alignment: Alignment.center,
        child: const CircularProgressIndicator(),
      );
    }
    return Scaffold(
        body: SingleChildScrollView(
            controller: _scrollController,
            child: Column(
              children: [
                SizedBox(height: size.height * 0.05),
                Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: [
                    IconButton(
                      icon: const Icon(
                        Icons.logout,
                        size: 30.0,
                      ),
                      padding: const EdgeInsets.only(left: 10),
                      onPressed: () {
                        Navigator.popUntil(context, (route) => route.isFirst);
                        Provider.of<AppState>(context, listen: false)
                            .setToken("");
                      },
                    ),
                    const Spacer(),
                    Container(
                      padding: const EdgeInsets.only(right: 20),
                      child: const BuildInfo(),
                    )
                  ],
                ),
                Container(
                    padding: const EdgeInsets.symmetric(horizontal: 40),
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 12),
                          child: CircleAvatar(
                              radius: 20,
                              backgroundImage: NetworkImage(_avatar)),
                        ),
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            Text(
                              _username,
                              style: const TextStyle(
                                  fontWeight: FontWeight.bold,
                                  color: Color(0xFF2661FA),
                                  fontSize: 20),
                            ),
                            const Text("Was möchtest du tun?"),
                          ],
                        ),
                      ],
                    )),
                SizedBox(height: size.height * 0.02),
                ManualInput(cb: onScan),
                bigButton("Bulk checkin",
                    () => Navigator.pushNamed(context, "/bulkCheckin")),
                bigButton("Bulk checkout",
                    () => Navigator.pushNamed(context, '/bulkCheckout')),
                bigButton(
                    "Audit", () => Navigator.pushNamed(context, '/auditAsset')),
                bigButton("Search asset", () async {
                  final result =
                      await Navigator.pushNamed(context, '/searchAsset');
                  if (!mounted) return;
                  if (result != null) {
                    onScan("$result");
                  }
                }),
                UserSelector(
                    cb: (user) => Navigator.pushNamed(context, "/userDetail",
                        arguments: user)),
              ],
            )));
  }
}
