import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/build_info.dart';
import 'package:snipe_scanner/main.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/model/login_user.dart';
import 'package:snipe_scanner/service/login_user.dart';
import 'package:snipe_scanner/utils/api_key_encryption.dart';
import 'package:snipe_scanner/utils/audio.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

class LoginWidget extends StatefulWidget {
  const LoginWidget({super.key});

  @override
  State<StatefulWidget> createState() => LoginWidgetState();
}

class LoginWidgetState extends ScanAwareState<LoginWidget> {
  String dropdownValue = 'Choose...';
  final List<String> _users = ['Choose...'];
  List<LoginUser> users = [];
  late TextEditingController _pinController;
  bool loading = true;
  String _errorText = '';

  @override
  void initState() {
    _pinController = TextEditingController();
    _pinController.clear();
    getUsers();
    super.initState();
  }

  @override
  void dispose() {
    _pinController.dispose();
    super.dispose();
  }

  @override
  void onScan(String barcode) {
    playSuccess();
    if (barcode != "") {
      _login(barcode);
    }
  }

  void resetState() {
    setState(() {
      _pinController.clear();
      _errorText = "";
      dropdownValue = "Choose...";
    });
  }

  @override
  void loadData() {
    getUsers();
    resetState();
  }

  void getUsers() async {
    if (service.get('host') == "") {
      setState(() {
        loading = false;
        _errorText =
            "You need to set the Address to your SnipeIT instance in the settings.";
      });
      return;
    }
    setState(() {
      loading = true;
      _errorText = "";
    });
    try {
      users = await getLoginUsers();
    } on ClientException catch (err) {
      if (err.message == "Connection refused") {
        setState(() {
          _errorText = "The connection to get the user list was refused.";
          loading = false;
        });
        return;
      }
      if (err.message == "Connection timed out") {
        setState(() {
          _errorText = "The connection to get the user list has timed out.";
          loading = false;
        });
        return;
      }
      rethrow;
    } on HttpException catch (err) {
      var errorText = "An unexpected error occured.";
      if (err.code == 404) {
        errorText = "Userlist was not found.";
      }
      setState(() {
        _errorText = errorText;
        loading = false;
      });
      return;
    }
    _users.clear();
    _users.add("Choose...");
    setState(() {
      _users.addAll(users.map<String>((LoginUser value) {
        return value.name;
      }));
      loading = false;
    });
  }

  void _login(String token) async {
    Provider.of<AppState>(context, listen: false).setToken(token);
    try {
      await getUser(token);
    } on HttpException catch (err) {
      var message = "Ein Fehler ist aufgetreten: $err";
      if (err.code == 401) {
        message = "Ungültiger Token";
      }
      setState(() {
        _errorText = "$message ${err.body}";
      });
      return;
    } on ArgumentError catch (err) {
      var message = "Ein Fehler ist aufgetreten: $err";
      setState(() {
        _errorText = message;
      });
      return;
    } catch (err) {
      setState(() {
        _errorText = err.toString();
      });
      return;
    }
    if (context.mounted) Navigator.pushNamed(context, "/dashboard");
  }

  void onEnter(string) {
    login();
  }

  void login() async {
    setState(() {
      _errorText = '';
    });
    LoginUser? user;
    for (var u in users) {
      if (u.name == dropdownValue) {
        user = u;
      }
    }
    if (user == null) {
      return;
    }
    String token = '';
    try {
      token = ApiKeyEncryption.decryptKey(user.token, _pinController.text);
    } catch (err) {
      setState(() {
        _errorText = "Falsche Pin";
      });
      return;
    }
    _login(token);
  }

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Scaffold(
        body: SingleChildScrollView(
            child: Column(
      children: [
        SizedBox(height: size.height * 0.05),
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            IconButton(
              icon: const Icon(
                Icons.settings,
                size: 30.0,
              ),
              padding: const EdgeInsets.all(0),
              onPressed: () {
                Navigator.pushNamed(context, '/preferences');
              },
            ),
            const Spacer(),
            Container(
              padding: const EdgeInsets.only(right: 20),
              child: const BuildInfo(),
            )
          ],
        ),
        SizedBox(height: size.height * 0.07),
        Container(
          alignment: Alignment.centerLeft,
          padding: const EdgeInsets.symmetric(horizontal: 40),
          child: const Text(
            "SnipeIT Scanner",
            style: TextStyle(
                fontWeight: FontWeight.bold,
                color: Color(0xFF133085),
                fontSize: 20),
            textAlign: TextAlign.left,
          ),
        ),
        Container(
          alignment: Alignment.centerLeft,
          padding: const EdgeInsets.symmetric(horizontal: 40),
          child: const Text(
            "LOGIN",
            style: TextStyle(
                fontWeight: FontWeight.bold,
                color: Color(0xFF2661FA),
                fontSize: 36),
            textAlign: TextAlign.left,
          ),
        ),
        if (_errorText != '')
          Container(
              padding: const EdgeInsets.only(top: 20),
              child: Text(
                _errorText,
                style: const TextStyle(
                    color: Colors.red, fontWeight: FontWeight.bold),
              )),
        SizedBox(height: size.height * 0.03),
        Container(
          alignment: Alignment.centerLeft,
          margin: const EdgeInsets.symmetric(horizontal: 40),
          child: Row(
            children: [
              DropdownButton<String>(
                value: dropdownValue,
                icon: const Icon(Icons.arrow_downward_sharp),
                elevation: 16,
                underline: Container(
                  height: 1,
                  color: Colors.grey,
                ),
                onChanged: (String? newValue) {
                  setState(() {
                    dropdownValue = newValue!;
                  });
                },
                items: _users.map<DropdownMenuItem<String>>((String value) {
                  return DropdownMenuItem<String>(
                    value: value,
                    child: Text(value),
                  );
                }).toList(),
              ),
              Container(
                padding: const EdgeInsets.fromLTRB(12, 0, 0, 0),
                height: 40,
                child: loading
                    ? const CircularProgressIndicator()
                    : ElevatedButton(
                        onPressed: () {
                          getUsers();
                        },
                        child: Container(
                          alignment: Alignment.center,
                          padding: const EdgeInsets.all(0),
                          child: const Icon(Icons.refresh),
                        ),
                      ),
              ),
            ],
          ),
        ),
        SizedBox(height: size.height * 0.03),
        Container(
          alignment: Alignment.center,
          margin: const EdgeInsets.symmetric(horizontal: 40),
          child: TextField(
            decoration: const InputDecoration(labelText: "Pin"),
            keyboardType: TextInputType.number,
            obscureText: true,
            onSubmitted: onEnter,
            maxLength: 32,
            enabled: dropdownValue != 'Choose...',
            controller: _pinController,
          ),
        ),
        Container(
          alignment: Alignment.centerRight,
          margin: const EdgeInsets.symmetric(horizontal: 40, vertical: 10),
          child: ElevatedButton(
            onPressed: login,
            child: Container(
              alignment: Alignment.center,
              height: 50.0,
              width: size.width * 0.5,
              padding: const EdgeInsets.all(0),
              child: const Text(
                "LOGIN",
                textAlign: TextAlign.center,
                style: TextStyle(fontWeight: FontWeight.bold),
              ),
            ),
          ),
        ),
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 80),
          child: Divider(height: size.height * 0.16),
        ),
        const Text("Or Scan QR Code to Login")
      ],
    )));
  }
}
