import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:snipe_scanner/components/search_item_row.dart';
import 'package:snipe_scanner/components/text_row.dart';
import 'package:snipe_scanner/model/app_state.dart';
import 'package:snipe_scanner/model/asset.dart';
import 'package:snipe_scanner/model/user.dart';
import 'package:snipe_scanner/service/asset.dart';
import 'package:snipe_scanner/service/users.dart';
import 'package:snipe_scanner/utils/scan_aware_state.dart';

class UserGeneralView extends StatelessWidget {
  final User user;
  const UserGeneralView({super.key, required this.user});

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.only(bottom: 20),
          height: 160,
          child: Center(
              child: CircleAvatar(
                  radius: 60, backgroundImage: NetworkImage(user.avatar))),
        ),
        Expanded(
          child: Scrollbar(
            child: Column(
              children: [
                TextRow(title: "Name", text: user.name),
                TextRow(title: "Title", text: user.jobtitle ?? " - "),
                TextRow(title: "Department", text: user.department ?? " - "),
                TextRow(title: "Locale", text: user.locale ?? " - "),
                TextRow(title: "E-Mail", text: user.email ?? " - "),
                TextRow(title: "Notes", text: user.notes ?? " - "),
                TextRow(title: "Created at", text: user.createdAt.formatted),
                TextRow(
                    title: "Last login",
                    text: user.lastLogin?.formatted ?? " - "),
              ],
            ),
          ),
        ),
      ],
    );
  }
}

class UserAssetsView extends StatelessWidget {
  final List<Asset> assets;
  const UserAssetsView({super.key, required this.assets});

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Column(
      children: [
        SizedBox(height: size.height * 0.03),
        Container(
          padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 20),
          child: const Text(
            "Assets",
            style: TextStyle(fontWeight: FontWeight.bold, fontSize: 24),
          ),
        ),
        Expanded(
          child: Scrollbar(
              child: ListView.builder(
                  shrinkWrap: true,
                  itemCount: assets.length,
                  itemBuilder: (BuildContext context, int index) {
                    var i = assets.length - (index + 1);
                    return searchItemRow(assets[i], context,
                        cb: (asset) => Navigator.pushNamed(
                            context, "/assetDetail",
                            arguments: asset.assetTag));
                  })),
        )
      ],
    );
  }
}

class UserDetail extends StatefulWidget {
  const UserDetail({super.key});

  @override
  ScanAwareState<UserDetail> createState() => _UserDetailState();
}

class _UserDetailState extends ScanAwareState<UserDetail> {
  int _index = 0;
  String _error = "";
  bool _loaded = false;
  User? _user;

  @override
  void loadData() async {
    final args = ModalRoute.of(context)!.settings.arguments as int;
    var token = Provider.of<AppState>(context, listen: false).token;

    setState(() {
      _loaded = false;
      _error = "";
    });
    User user;
    try {
      user = await getUser(token, args);
    } catch (err) {
      setState(() {
        _error = err.toString();
        _loaded = true;
      });
      return;
    }
    //if (user.avatar != null) {
    if (user.avatar.startsWith("//")) {
      user.avatar = "https:${user.avatar}";
    }
    if (context.mounted) {
      await precacheImage(NetworkImage(user.avatar), context);
    }
    //}
    setState(() {
      _user = user;
      _loaded = true;
    });
  }

  Widget _selector(BuildContext context) {
    if (_index == 1) {
      return FutureBuilder<List<Asset>>(
          future: getUserAssets(
              Provider.of<AppState>(context, listen: false).token, _user!.id),
          builder: (BuildContext context, AsyncSnapshot<List<Asset>> snapshot) {
            if (snapshot.hasData) {
              return UserAssetsView(assets: snapshot.data!);
            } else if (snapshot.hasError) {
              return Container(
                  padding:
                      const EdgeInsets.symmetric(horizontal: 40, vertical: 40),
                  child: Text(
                    _error,
                    style: const TextStyle(fontSize: 20),
                  ));
            } else {
              return const Center(child: CircularProgressIndicator());
            }
          });
    }

    return UserGeneralView(user: _user!);
  }

  @override
  Widget build(BuildContext context) {
    if (!_loaded) {
      return const Center(
        child: CircularProgressIndicator(),
      );
    }
    if (_error != "") {
      return Scaffold(
          appBar: AppBar(
            title: const Text("Fehler"),
            actions: [
              IconButton(
                  icon: const Icon(Icons.refresh),
                  tooltip: 'Reload',
                  onPressed: () => loadData()),
            ],
          ),
          body: Container(
            padding: const EdgeInsets.symmetric(horizontal: 40, vertical: 40),
            child: Text(
              _error,
              style: const TextStyle(fontSize: 20),
            ),
          ));
    }
    return Scaffold(
        appBar: AppBar(
          title: Text(_user!.name),
          actions: [
            IconButton(
                icon: const Icon(Icons.refresh),
                tooltip: 'Reload',
                onPressed: () {
                  loadData();
                }),
          ],
        ),
        bottomNavigationBar: BottomNavigationBar(
          currentIndex: _index,
          items: const <BottomNavigationBarItem>[
            BottomNavigationBarItem(
              icon: Icon(Icons.home),
              label: 'General',
            ),
            BottomNavigationBarItem(
              icon: Icon(Icons.list),
              label: 'Assets',
            ),
          ],
          onTap: (value) {
            setState(() {
              _index = value;
            });
          },
        ),
        body: _selector(context));
  }
}
